%VAR_for_MOG.m
%VAR modeling by particle filtering for MoG TVAR processes
%by Deniz Gencaga, 2007.
% Please cite the following publication if you are inspired from this method:
% D. Gencaga, E. E. Kuruoglu, A. Ertuzun, “Modeling non-Gaussian time-varying 
%vector autoregressive processes by particle filtering”, Multidimensional 
%Systems and Signal Processing, Vol. 21, Number 1, pp. 73-85, March 2010.


clear all;

tic;


M = 1000;%data length

numSamples = 100;%number of samples

lambda = 0.9;%forgetting factor

%SYNTHETIC DATA GENERATION
%ELEMENTS OF TVAR MATRIX
a1 = [0.5*ones(1,M/2),-0.5*ones(1,M/2)];
b = [0.1*ones(1,M/2),-0.1*ones(1,M/2)];
c = [0.4*ones(1,M/2),-0.4*ones(1,M/2)];
d = [0.5*ones(1,M/2),-0.5*ones(1,M/2)];

ensemble_no = 100;%ENSEMBLE SIZE

a_cum = zeros(4,M);

s_track = zeros(2,M,ensemble_no);
n_track = zeros(1,M,ensemble_no);
v_track = zeros(1,M,ensemble_no);

a_ens = zeros(4,M,ensemble_no);


for iteration_no = 1:ensemble_no;
    
    iteration_no
    
    seed = sum(100*clock);

    rand('state',seed)
    randn('state',seed)




%MOG DATA GENERATION
s = rand(2,M);

for i = 2:M;
    
    r = rand;
    
    if r <=0.5,
        
        n = sqrt(1)*randn;
        v = sqrt(1)*randn ;
        
    else
        
        n = 1+sqrt(.1)*randn ;
        v = 1+sqrt(.1)*randn ;
        
    end
    
    n_track(1,i,iteration_no) = n;
    v_track(1,i,iteration_no) = v;
   
    
    s(:,i) =   [a1(i) b(i);c(i) d(i)] * s(:,i-1) + [n;v];
    
    s_track(:,i,iteration_no) = s(:,i);%TVAR PROCESSES
    
end




%%%%%%%%%%%%%%%%%% PARTICLE FILTER STARTS HERE%%%%%%%%%%%%%%%%%%%%%%%%%%5

%INITIATION OF AR PARTICLES
a = randn(4,numSamples);

resampled_a = zeros(size(a));



a_track = zeros(4,M);

for i=2:M;
    

    %TRANSITION OF AR PARTICLES
    sigma_a = var(a');
    
    sigma_u = diag(sigma_a) * (1/lambda - 1);
    
    a = a + sqrt(sigma_u) * randn(4,numSamples);
    
    w = zeros(1,numSamples);%IMPORTANCE WEIGTHS
    
    for j=1:numSamples;
        
        deger =  [a(1,j) a(2,j);a(3,j) a(4,j)] * s(:,i-1);

        
        w(j) = 0.5 * gauss_kernel(deger',[1 0;0 1],s(:,i)') + 0.5 * gauss_kernel(deger'+[1 1],[0.1 0;0 0.1],s(:,i)') ;%IMP. WEIGHT CALCULATION
        
    end
    
    w = w./sum(w);%NORMALIZATION
    
    
        %%%%%%%%%%%%%%%%%%%%%%%%%%Resampling%%%%%%%%%%%%%%%%
     

        a(1,:)=Res_Res(a(1,:),w);
        a(2,:)=Res_Res(a(2,:),w);
        a(3,:)=Res_Res(a(3,:),w);
        a(4,:)=Res_Res(a(4,:),w);
        
        
        
        
        %%%%%%%%% MMSE ESTIMATES%%%%%%%%%%%%%%%%%%%%%%

        a_track(1,i) = (1/numSamples) * sum(a(1,:));
        a_track(2,i) = (1/numSamples) * sum(a(2,:));
        a_track(3,i) = (1/numSamples) * sum(a(3,:));
        a_track(4,i) = (1/numSamples) * sum(a(4,:));    
        
        %%%%%%%%%%%%%STORING ESTIMATES IN AN ARRAY %%%%%%%%%%%
        
        a_ens(1,i,iteration_no) = a_track(1,i);
        a_ens(2,i,iteration_no) = a_track(2,i);
        a_ens(3,i,iteration_no) = a_track(3,i);
        a_ens(4,i,iteration_no) = a_track(4,i);

        
end

a_cum = a_cum + a_track;

end

a_cum = a_cum/ensemble_no;%ENSEMBLE AVERAGE

tt = toc;

%%%%%%%%%%%%%%%PLOTTING
figure;

subplot(211);
plot(s(1,:));
subplot(212);
plot(s(2,:));


figure;

subplot(221);
plot(a_cum(1,1:M));
hold on;
plot(a1,'r');
hold off;
subplot(222);
plot(a_cum(2,1:M));
hold on;
plot(b,'r');
hold off;

subplot(223);
plot(a_cum(3,1:M));
hold on;
plot(c,'r');
hold off;

subplot(224);
plot(a_cum(4,1:M));
hold on;
plot(d,'r');
hold off;


    
    
    

